/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 * Description: 通讯组件（CM）event异步调用 示例程序客户端头文件
 */

#ifndef HUAWEI_AP_CM_EVENT_DDS_ASYNC_CLIENT_ACTIVITY_H
#define HUAWEI_AP_CM_EVENT_DDS_ASYNC_CLIENT_ACTIVITY_H

// 建议包含生成代码中的executable.h(内部包含了xxx_proxy.h)
#include "visualization/executable.h"
#include "ara/log/logging.h"
#include <publisher.h>

namespace sample
{
	class ClientActivity
	{
		using ProxySocA = huawei::ap::sample::proxy::VisualizationProxy;
		//using ProxySocB = huawei::ap::sample::proxy::VisualizationSoCBProxy;

	public:
		static enum SensorAIndexes {
			CAMERA_A,
			LIDAR_INNO,
			LIDAR_INNO2,
			HESAI_LEFT,
			HESAI_LEFT2,
			HESAI_RIGHT,
			HESAI_RIGHT2,
			SENSORS_A_COUNT
		};

		static enum SensorBIndexes {
			CAMERA_FE,
			LIDAR_INNO3,
			LIDAR_INNO4,
			SENSORS_B_COUNT
		};

		static enum HesaiLidarIndex {
			HESAI_LEFT_IDX,
			HESAI_LEFT2_IDX,
			HESAI_RIGHT_IDX,
			HESAI_RIGHT2_IDX,
			HESAI_COUNT

		};

		static enum InnoLidarIndex {
			LIDAR_INNO_IDX,
			LIDAR_INNO2_IDX,
			LIDAR_INNO3_IDX,
			LIDAR_INNO4_IDX,
			INNO_COUNT
		};

		ClientActivity();
		~ClientActivity();
		void Init();
		void Act();
		void Stop();

	protected:
		void setupProxies();

		//    std::shared_ptr<huawei::ap::sample::skeleton::VisualizationSkeleton> skeleton_;
		void CameraA_setupPublishers();
		void CameraA_setupFindService();
		void CameraA_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
												 ara::com::FindServiceHandle handler);
		void CameraA_ImageCallback();
		void CameraA_stop();



		void HesaiLidarRight1_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
														  ara::com::FindServiceHandle handler);
		void HesaiLidarRight1_DataCallback();
		void HesaiLidarRight1_setupPublishers();
		void HesaiLidarRight1_setupFindService();
		void HesaiLidarRight1_stop();

		void HesaiLidarRight2_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
														  ara::com::FindServiceHandle handler);
		void HesaiLidarRight2_DataCallback();
		void HesaiLidarRight2_setupPublishers();
		void HesaiLidarRight2_setupFindService();
		void HesaiLidarRight2_stop();

		void HesaiLidarLeft1_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
														 ara::com::FindServiceHandle handler);
		void HesaiLidarLeft1_DataCallback();
		void HesaiLidarLeft1_setupPublishers();
		void HesaiLidarLeft1_setupFindService();
		void HesaiLidarLeft1_stop();

		void HesaiLidarLeft2_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
														 ara::com::FindServiceHandle handler);
		void HesaiLidarLeft2_DataCallback();
		void HesaiLidarLeft2_setupPublishers();
		void HesaiLidarLeft2_setupFindService();
		void HesaiLidarLeft2_stop();

		ara::core::Vector<mdc::visual::PointField> InnoCommon_ConstructPointField();
		void InnoLidarFront1_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
														 ara::com::FindServiceHandle handler);
		void InnoLidarFront1_DataCallback();
		void InnoLidarFront1_setupPublishers();
		void InnoLidarFront1_setupFindService();
		void InnoLidarFront1_stop();
		void InnoLidarFront1_initPointCloud2();
		void InnoLidarFront1_checkPc2Queue();
		void InnoLidarFront1_runThread();

		void InnoLidarFront2_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
													 ara::com::FindServiceHandle handler);
		void InnoLidarFront2_DataCallback();
		void InnoLidarFront2_setupPublishers();
		void InnoLidarFront2_setupFindService();
		void InnoLidarFront2_stop();
		void InnoLidarFront2_initPointCloud2();
		void InnoLidarFront2_checkPc2Queue();
		void InnoLidarFront2_runThread();



		std::shared_ptr<ProxySocA> proxySocA_[SENSORS_A_COUNT];
		std::mutex proxySocAMutex_[SENSORS_A_COUNT];



		std::uint32_t validDataCount_;
		ara::com::FindServiceHandle serviceHandleA[SENSORS_A_COUNT];
		ara::com::FindServiceHandle serviceHandleB[SENSORS_B_COUNT];
		ara::log::Logger &log_{ara::log::CreateLogger("act", "cm dds event async client sample context",
													  ara::log::LogLevel::kVerbose)};

	private:
		uint8_t soc_a_cam_idx_div_factor = 8;
		uint8_t soc_b_cam_idx_div_factor = 4;

		std::string topic_prefix = "Camera_";
		mdc::visual::Publisher CameraA_imagePub[4] = {};
		mdc::visual::Publisher CameraFE_imagePub[4] = {};
		mdc::visual::Publisher innoPub_[INNO_COUNT] = {};
		mdc::visual::Publisher hesaiPub_[HESAI_COUNT] = {};

		// mdc::visual::Image image;
		std::string frame_name = "iv_points";
		int width = 0;

		bool isInitialized{false};
		std::queue<mdc::visual::PointCloud2> InnoLidarFront_pc2Queue[INNO_COUNT];
		std::thread InnoLidarFront_pc2Thread[INNO_COUNT];
		void InnoLidar_checkPc2Queue();
	};
} // namespace camera

#endif // HUAWEI_AP_CM_EVENT_DDS_ASYNC_CLIENT_ACTIVITY_H
